/**
 * @file
 * A Backbone view for the edit_ui block element.
 */

(function (Drupal, Backbone, $) {
  "use strict";

  var strings = {
    confirmDelete: Drupal.t('Are you sure you want to delete the block "@name"?')
  };

  /**
   * Backbone view for the edit_ui block.
   */
  Drupal.editUiBlock.BlockVisualView = Backbone.View.extend({
    /**
     * Custom data.
     */
    hiddenClass: 'visually-hidden',

    /**
     * Dom elements events.
     */
    events: {
      "mouseenter": "select",
      "mouseleave": "unselect",
      "mousedown": "startDrag",
      "mousedown .contextual": function(e){e.stopPropagation();},
      "click .edit-uiblockdelete": "deleteBlock"
    },

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      this.listenTo(this.model, 'change:isSelected', this.render);
      this.listenTo(this.model, 'change:isDragging', this.render);
      this.listenTo(this.model, 'change:region', this.move);
      this.listenTo(this.model, 'destroy', this.remove);

      // Init data.
      this.$ghost = $(Drupal.theme('editUiGhostBlock'));

      // Init model state.
      this.model.setBlock(this.$el);
      this.model.visible();
    },

    /**
     * {@inheritdoc}
     */
    render: function() {
      if (this.model.get('isDragging')) {
        this.$el.addClass(this.hiddenClass);
        this.$ghost.insertAfter(this.$el);
        this.$ghost.css({
          width: this.model.get('width'),
          height: this.model.get('height')
        });
      } else {
        this.$el.removeClass(this.hiddenClass);
        this.$ghost.detach();
      }
    },

    /**
     * Move block position.
     */
    move: function() {
      var beforeBlock;
      var cid = this.model.cid;
      var weight = this.model.get('weight');
      var region = this.model.get('region');
      var blocks = Drupal.editUiBlock.collections.blockCollection.where({region: region});

      // Find the block just before.
      blocks.forEach(function(block){
        if (block.get('isVisible') && block.get('weight') <= weight && block.cid != cid) {
          beforeBlock = block;
        }
      });

      // If not get the region.
      if (!beforeBlock) {
        beforeBlock = Drupal.editUiRegion.collections.regionCollection.findWhere({region: region});
      }

      // Insert block.
      beforeBlock.get('block').after(this.$el);
    },

    /**
     * Set block selected.
     */
    select: function() {
      this.model.select();
    },

    /**
     * Set block not selected.
     */
    unselect: function() {
      this.model.unselect();
    },

    /**
     * Start drag.
     */
    startDrag: function(event) {
      if (!Drupal.editUiToolbar.models.toolbarModel.get('isOpen')) {
        return;
      }
      event.preventDefault();

      // Update model.
      this.model.startDrag(
        this.$el.outerWidth(),
        this.$el.outerHeight(),
        this.$el.offset()
      );
    },

    /**
     * Delete block.
     *
     * @param {Event} event
     *   Event object.
     */
    deleteBlock: function(event) {
      event.preventDefault();
      var message = Drupal.formatString(strings.confirmDelete, {'@name': this.model.get('label')});
      if (confirm(message)) {
        this.model.destroy();
      }
    }
  });

  /**
   * Ghost block template.
   *
   * @return String
   *   A string representing a DOM fragment.
   */
  Drupal.theme.editUiGhostBlock = function () {
    return '<div class="edit-ui__ghost-block"></div>';
  };

}(Drupal, Backbone, jQuery));
