/**
 * @file
 * A Backbone view for the edit_ui body element.
 */

(function (Drupal, Backbone, $) {
  "use strict";

  /**
   * Backbone view for the edit_ui body.
   */
  Drupal.editUiBlock.BodyVisualView = Backbone.View.extend({
    /**
     * Main element.
     */
    el: 'body',

    /**
     * Dom elements events.
     */
    events: {},

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      $(document).on('drupalEditUiBlockAdded.editUiBlockBodyVisualView', this.editUiBlockAdded.bind(this));

      // Get data.
      this.$draggingBlock = $('#edit-ui__dragging-block');
    },

    /**
     * {@inheritdoc}
     */
    remove: function() {
      this.$el.off('.editUiBlockPageVisualView');
      $(document).off('.editUiBlockPageVisualView');
      Backbone.View.prototype.remove.apply(this, arguments);
    },

    /**
     * {@inheritdoc}
     */
    render: function(model) {
      if (model.get('isDragging')) {
        this.startDrag(model);
      } else {
        this.stopDrag();
      }
    },

    /**
     * Edit_ui block added callback.
     *
     * @param {Event} event
     *   Event object.
     * @param {Object} data
     *   Callback data.
     */
    editUiBlockAdded: function(event, data) {
      this.listenTo(data.model, 'change:isDragging', this.render);
    },

    /**
     * Reset dragging block style.
     */
    resetStyle: function() {
      this.$draggingBlock.css({
        width: '',
        height: '',
        marginTop: '',
        marginLeft: '',
        borderRadius: '',
        opacity: ''
      });
    },

    /**
     * Start drag.
     *
     * @param {Drupal.editUiBlock.BlockModel} block
     *   The block model of the dragged element.
     */
    startDrag: function(block) {
      this.$draggingBlock.css({
        width: block.get('width'),
        height: block.get('height'),
        top: event.pageY - 20,
        left: event.pageX - 20,
        marginTop: - event.pageY + block.get('offset').top + 20,
        marginLeft: - event.pageX + block.get('offset').left + 20,
        borderRadius: 0,
        opacity: 0
      });
      this.$draggingBlock.show();

      // Start dragging animation.
      setTimeout(this.resetStyle.bind(this), 0);

      // Initialize regions data.
      Drupal.editUiRegion.collections.regionCollection.init();

      // Add listeners.
      this.$el.on('mousemove.editUiBlockPageVisualView', $.proxy(this.drag, this, block));
      this.$el.on('mouseup.editUiBlockPageVisualView', $.proxy(this.drop, this, block));
    },

    /**
     * Drag.
     *
     * @param {Drupal.editUiBlock.BlockModel} block
     *   The dropped block.
     * @param {Event} event
     *   The event object.
     */
    drag: function(block, event) {
      this.$draggingBlock.css({
        top: event.pageY - 20,
        left: event.pageX - 20
      });
      Drupal.editUiRegion.collections.regionCollection.drag(event.pageX, event.pageY);
    },

    /**
     * Drop.
     *
     * @param {Drupal.editUiBlock.BlockModel} block
     *   The dropped block.
     * @param {Event} event
     *   The event object.
     */
    drop: function(block, event) {
      Drupal.editUiRegion.collections.regionCollection.drop(block, event.pageX, event.pageY);
    },

    /**
     * Stop drag.
     */
    stopDrag: function() {
      this.$draggingBlock.hide();

      // Remove listeners.
      this.$el.off('mousemove.editUiBlockPageVisualView');
      this.$el.off('mouseup.editUiBlockPageVisualView');
    }
  });

}(Drupal, Backbone, jQuery));
